module edge_detector(sign_i, // Input of the Edge Detector
rise_o, // Rising Edge Output of the Edge Detector
fall_o, // Falling Edge Output of the Edge Detector
clk, // clock
rst_b, // Synchronous Reset
);

//--------------------- Parameters Declaration ---------------------------------
parameter SIZE = 1;

//-------------------- Inputs Declaration --------------------------------------
input [SIZE-1:0]sign_i;
input clk;
input rst_b;


//---------------------- Outputs Declaration -----------------------------------
output [SIZE-1:0]rise_o;
output [SIZE-1:0]fall_o;

//------------------- Types Declaration ----------------------------------------
wire clk;
wire rst_b;
wire [SIZE-1:0]sign_i;
wire [SIZE-1:0]rise_o;
wire [SIZE-1:0]fall_o;
reg [SIZE-1:0]sign_i_syn;
reg [SIZE-1:0]out_ff2;
wire [SIZE-1:0]out_xor;

genvar i; // auxiliar variable to generate

generate for(i=0;i < SIZE; i=i+1 )
begin: genseq
//------------------------ Sequential Part ------------------------------------ 

always @(posedge clk)
begin
if (!rst_b)
begin

sign_i_syn[i]<=1'b0; //If reset occured clear flip flops

out_ff2<=1'b0; 

end //if(!rst) 

else 
begin 

sign_i_syn[i]<= sign_i[i];
//Transform the entering signal in synchronous 

out_ff2 [i] <= sign_i_syn[i];
//Update the second f! f output value

end //else 
end //always @(posedge clk)


//-------------------------------- Combinational Part ------------------------ 

assign out_xor[i] = (sign_i_syn[i] ^ out_ff2[i]); 
// keeps the output from the xor in 1 for 1 period of clock in the rise or fall

assign rise_o[i] = (sign_i_syn[i] & out_xor[i]); 
// if the entering sinal has high level and the pulse in the xor output 
// represents a rising edge

assign fall_o[i] = ~(sign_i_syn[i] | ~out_xor[i]); 
// if the entering sinal has low level and the pulse in the xor output 
// represents a falling edge

end // for (generate) 
endgenerate

  generate for(i=0;i < SIZE; i=i+1)
   begin: Rise   
     // psl sequence PULSE(boolean a) = { a; !a };
     //psl RISING: assert always ( rose(sign_i[0]) -> next PULSE(rise_o)    
     // abort !rst_b)   
     // report "Rising Edge Failed " severity warning;   
     //@rose(clk);  
     
   end
 endgenerate 


endmodule

module testit;

  reg clk = 0;
  always #5 clk = !clk;
  reg rst_b = 0;
  initial #10 rst_b = 1;
  
  reg sign;
  wire rise, fall;
  
  initial
  begin
    sign = 0;
    #20 sign = 1;
    #20 sign = 0;
    #20 sign = 1;
    #20 $finish;
  end

edge_detector ed (sign, // Input of the Edge Detector
     rise, // Rising Edge Output of the Edge Detector
     fall, // Falling Edge Output of the Edge Detector
     clk, // clock
     rst_b // Synchronous Reset
     );


endmodule
